"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const shelljs_1 = __importDefault(require("shelljs"));
const __1 = require("../..");
exports.command = 'plugin <name>';
exports.desc = 'Generate a plugin structure';
exports.builder = function (yargs) {
    yargs.option('force', {
        describe: 'force creation, remove existed one',
        alias: 'f'
    });
};
exports.handler = function (argv) {
    let pluginDir = argv.pluginMakeDir || argv.pluginDir;
    if (!pluginDir || !__1.Utils.fileExistsSyncCache(pluginDir)) {
        __1.Utils.error('"pluginDir" missing in config file or not exist in current directory!');
    }
    const namePattern = /[a-z0-9]+/;
    if (!namePattern.test(argv.name)) {
        __1.Utils.error('Plugin name invalid!');
    }
    const scriptName = argv.scriptName || 'zignis';
    const pluginPath = path_1.default.resolve(pluginDir, `${scriptName}-plugin-${argv.name}`);
    if (__1.Utils.fileExistsSyncCache(pluginPath)) {
        if (argv.force) {
            __1.Utils.warn(`Existed ${scriptName}-plugin-${argv.name} is deleted before creating a new one!`);
            shelljs_1.default.rm('-rf', pluginPath);
        }
        else {
            __1.Utils.error(`Destination existed, command abort!`);
        }
    }
    shelljs_1.default.mkdir('-p', pluginPath);
    shelljs_1.default.cd(pluginPath);
    if (!shelljs_1.default.which(scriptName)) {
        __1.Utils.error(`Script ${scriptName} not found!`);
    }
    shelljs_1.default.exec('npm init --yes');
    shelljs_1.default.exec(`${scriptName} init --plugin --exec-mode`, (code, stdout, stderr) => { });
};
//# sourceMappingURL=plugin.js.map