"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = __importDefault(require("lodash"));
const dayjs_1 = __importDefault(require("dayjs"));
const __1 = require("../..");
exports.command = 'script <name>';
exports.desc = 'Generate a script file';
exports.builder = function (yargs) {
    yargs.option('typescript', {
        alias: 'ts',
        describe: 'generate typescript style code'
    });
};
exports.handler = function (argv) {
    let scriptDir = argv.scriptMakeDir || argv.scriptDir;
    if (!scriptDir || !__1.Utils.fileExistsSyncCache(scriptDir)) {
        console.log(chalk_1.default.red('"scriptDir" missing in config file or not exist in current directory!'));
        return;
    }
    const filePrefix = dayjs_1.default().format('YYYYMMDDHHmmssSSS');
    const scriptFile = path_1.default.resolve(scriptDir, `${filePrefix}_${lodash_1.default.kebabCase(argv.name)}.${argv.typescript ? 'ts' : 'js'}`);
    if (__1.Utils.fileExistsSyncCache(scriptFile)) {
        console.log(chalk_1.default.red('Scritp file exist!'));
        return;
    }
    let code;
    if (argv.typescript) {
        code = `export const builder = function (yargs: any) {
  // yargs.option('option', {default, describe, alias})
}

export const handler = async function (argv: any) {
  console.log('Start to draw your dream code!')
  process.exit(0)
}
`;
    }
    else {
        code = `exports.builder = function (yargs) {
  // yargs.option('option', {default, describe, alias})
}

exports.handler = async function (argv) {
  console.log('Start to draw your dream code!')
  process.exit(0)
}
`;
    }
    if (!__1.Utils.fileExistsSyncCache(scriptFile)) {
        fs_1.default.writeFileSync(scriptFile, code);
        console.log(chalk_1.default.green(`${scriptFile} created!`));
    }
};
//# sourceMappingURL=script.js.map