"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const shelljs_1 = __importDefault(require("shelljs"));
const __1 = require("..");
exports.command = 'new <name> [repo] [branch]';
exports.aliases = 'n';
exports.desc = 'Create a new project from specific repo';
exports.builder = function (yargs) {
    yargs.option('yarn', {
        default: false,
        describe: 'use yarn command'
    });
    yargs.option('yes', {
        default: true,
        alias: 'y',
        describe: 'run npm/yarn init with --yes'
    });
    yargs.option('force', {
        alias: 'f',
        describe: 'force download, existed folder will be deleted!'
    });
    yargs.option('merge', {
        alias: 'm',
        describe: 'merge config with exist project folder!'
    });
    yargs.option('empty', {
        alias: 'e',
        describe: 'force empty project, ignore repo'
    });
    yargs.option('select', {
        alias: 's',
        describe: 'select from default repos'
    });
    yargs.option('add', {
        default: false,
        alias: 'A',
        describe: 'add npm package to package.json dependencies'
    });
    yargs.option('add-dev', {
        default: false,
        alias: 'D',
        describe: 'add npm package to package.json devDependencies'
    });
    yargs.option('init', {
        alias: 'i',
        describe: 'init new project'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        argv.repo = argv.repo || '';
        argv.branch = argv.branch || 'master';
        try {
            if (__1.Utils.fileExistsSyncCache(path_1.default.resolve(process.cwd(), argv.name))) {
                if (argv.force) {
                    shelljs_1.default.rm('-rf', path_1.default.resolve(process.cwd(), argv.name));
                    __1.Utils.warn(`Existed ${argv.name} is deleted before creating a new one!`);
                }
                else if (!argv.merge) {
                    __1.Utils.error(`Destination existed, command abort!`);
                }
            }
            if (argv.merge) {
                // Nothing happened.
                shelljs_1.default.cd(argv.name);
            }
            else {
                if (argv.empty || !argv.repo) {
                    shelljs_1.default.mkdir('-p', path_1.default.resolve(process.cwd(), argv.name));
                    shelljs_1.default.cd(argv.name);
                    if (argv.yarn) {
                        if (argv.yes) {
                            __1.Utils.exec('yarn init -y');
                        }
                        else {
                            __1.Utils.exec('yarn init');
                        }
                    }
                    else {
                        if (argv.yes) {
                            __1.Utils.exec('npm init -y');
                        }
                        else {
                            __1.Utils.exec('npm init');
                        }
                    }
                    __1.Utils.exec(`echo "node_modules" > .gitignore`);
                    __1.Utils.exec('git init');
                    __1.Utils.success('New .git directory has been created!');
                }
                else {
                    if (argv.select) {
                        const defaultRepos = yield __1.Utils.invokeHook('new_repo');
                        if (Object.keys(defaultRepos).length === 0) {
                            __1.Utils.error('No pre-defined repos available.');
                        }
                        const select = defaultRepos[argv.select]
                            ? argv.select
                            : Object.keys(defaultRepos).find(key => defaultRepos[key].alias && defaultRepos[key].alias.indexOf(argv.select) > -1);
                        if (select && defaultRepos[select]) {
                            argv.repo = defaultRepos[select].repo || __1.Utils.error('Repo not found');
                            argv.branch = defaultRepos[select].branch || 'master';
                        }
                        else {
                            const answers = yield __1.Utils.inquirer.prompt([
                                {
                                    type: 'list',
                                    name: 'selected',
                                    message: `Please choose a pre-defined repo to continue:`,
                                    choices: Object.keys(defaultRepos).map(key => {
                                        return { name: `${key} [${defaultRepos[key].alias.join(', ')}]`, value: key };
                                    })
                                }
                            ]);
                            argv.repo = defaultRepos[answers.selected].repo || __1.Utils.error('Repo not found');
                            argv.branch = defaultRepos[answers.selected].branch || 'master';
                        }
                    }
                    __1.Utils.info(`Downloading from ${argv.repo}`);
                    try {
                        __1.Utils.exec(`git clone ${argv.repo} ${argv.name} --single-branch --depth=1 --branch ${argv.branch} --progress`);
                        __1.Utils.success('Succeeded!');
                        shelljs_1.default.cd(argv.name);
                        shelljs_1.default.rm('-rf', path_1.default.resolve(process.cwd(), `.git`));
                        __1.Utils.success('.git directory removed!');
                        if (argv.yarn) {
                            __1.Utils.exec('yarn');
                        }
                        else {
                            __1.Utils.exec('npm install');
                        }
                        __1.Utils.exec('git init');
                        __1.Utils.success('New .git directory has been created!');
                    }
                    catch (e) {
                        __1.Utils.error(e.message);
                    }
                }
            }
            // add packages
            const addPackage = __1.Utils.parsePackageNames(argv.add);
            const addPackageDev = __1.Utils.parsePackageNames(argv.addDev);
            if (addPackage.length > 0) {
                if (argv.yarn) {
                    __1.Utils.exec(`yarn add ${addPackage.join(' ')}`);
                }
                else {
                    __1.Utils.exec(`npm install ${addPackage.join(' ')}`);
                }
            }
            if (addPackageDev.length > 0) {
                if (argv.yarn) {
                    __1.Utils.exec(`yarn add ${addPackageDev.join(' ')} -D`);
                }
                else {
                    __1.Utils.exec(`npm install ${addPackageDev.join(' ')} --save-dev`);
                }
            }
            // init basic structure
            if (argv.init) {
                const initExtra = argv.yarn ? '--yarn' : '';
                if (argv.name.indexOf(`${argv.scriptName}-plugin-`) === 0) {
                    __1.Utils.exec(`${argv.scriptName} init --exec-mode --plugin --force ${initExtra}`);
                }
                else {
                    __1.Utils.exec(`${argv.scriptName} init --exec-mode --force  ${initExtra}`);
                }
                __1.Utils.success('Initial basic structure complete!');
            }
        }
        catch (e) {
            __1.Utils.error(e.stack);
        }
    });
};
//# sourceMappingURL=new.js.map