"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const repl_1 = __importDefault(require("repl"));
const lodash_1 = __importDefault(require("lodash"));
const repl_history_1 = __importDefault(require("repl.history"));
const __1 = require("..");
function corepl(cli) {
    var originalEval = cli.eval;
    // @ts-ignore
    cli.eval = function coEval(cmd, context, filename, callback) {
        if (cmd.trim() === '?') {
            console.log();
            __1.Utils.outputTable([
                ['quit', 'Quit the REPL, alias: exit, q.'],
                ['await', 'Run generator or promise function.'],
                ['?', 'Show this help info.']
            ], 'Internal commands:');
            console.log();
            return callback();
        }
        if (['exit', 'quit', 'q'].includes(cmd.replace(/(^\s*)|(\s*$)/g, ''))) {
            console.log(chalk_1.default.yellow('Bye!'));
            process.exit(0);
        }
        if (cmd.match(/^await\s+/) || cmd.match(/.*?await\s+/) && cmd.match(/^\s*\{/)) {
            if (cmd.match(/=/)) {
                cmd = '(async function() { (' + cmd + ') })()';
            }
            else {
                cmd = '(async function() { let _ = ' + cmd + '; return _;})()';
            }
        }
        else if (cmd.match(/\W*await\s+/)) {
            cmd = '(async function() { (' + cmd.replace(/^\s*(var|let|const)\s+/, '') + ') })()';
        }
        function done(val) {
            return callback(null, val);
        }
        originalEval.call(cli, cmd, context, filename, function (err, res) {
            if (err || !res || typeof res.then !== 'function') {
                return callback(err, res);
            }
            else {
                return res.then(done, callback);
            }
        });
    };
    return cli;
}
exports.command = 'repl';
exports.aliases = 'r';
exports.desc = 'Play with REPL';
function openRepl(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const { argv } = context;
        const r = repl_1.default.start({
            prompt: argv.prompt,
        });
        const Home = process.env.HOME + `/.${argv.scriptName}`;
        if (!__1.Utils.fileExistsSyncCache(Home)) {
            __1.Utils.exec(`mkdir -p ${Home}`);
        }
        repl_history_1.default(r, `${Home}/.${argv.scriptName}_history`);
        // @ts-ignore
        // context即为REPL中的上下文环境
        r.context = Object.assign(r.context, context);
        corepl(r);
    });
}
exports.builder = function (yargs) {
    yargs.option('hook', {
        describe: 'if or not load all plugins repl hook'
    });
    yargs.option('prompt', {
        default: '>>> ',
        describe: 'Prompt for input.'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        argv.hook = argv.hook || false;
        try {
            let context = { Utils: __1.Utils, argv, await: true };
            if (argv.hook) {
                const pluginsReturn = yield __1.Utils.invokeHook('repl', lodash_1.default.isBoolean(argv.hook)
                    ? {}
                    : {
                        include: __1.Utils.splitComma(argv.hook)
                    });
                context = Object.assign(context, pluginsReturn);
            }
            return yield openRepl(context);
        }
        catch (e) {
            __1.Utils.error(e.stack);
        }
    });
};
//# sourceMappingURL=repl.js.map