"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const __1 = require("..");
exports.command = 'status';
exports.aliases = 'st';
exports.desc = 'Show environment status info';
exports.builder = function (yargs) {
    yargs.option('list-plugin', {
        alias: ['plugin', 'P'],
        describe: 'Show valid plugin list'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const scriptName = argv.scriptName || 'zignis';
        try {
            // basic information
            const hookStatus = yield __1.Utils.invokeHook('status', { mode: 'group' });
            const columns = [];
            let kvs = hookStatus[scriptName] ? hookStatus[scriptName] : {};
            if (Object.keys(kvs).length > 0) {
                Object.keys(kvs).map(k => columns.push([k, kvs[k]]));
                __1.Utils.outputTable(columns, 'Core Information');
            }
            if (argv.listPlugin) {
                // plugin information
                const plugins = __1.Utils.getAllPluginsMapping();
                Object.keys(plugins).forEach(plugin => {
                    if (__1.Utils.fileExistsSyncCache(path_1.default.resolve(plugins[plugin], 'package.json'))) {
                        const pkgConfig = require(path_1.default.resolve(plugins[plugin], 'package.json'));
                        const pluginColumns = [];
                        if (pkgConfig.version) {
                            pluginColumns.push(['version', pkgConfig.version]);
                        }
                        if (hookStatus && hookStatus[plugin]) {
                            Object.keys(hookStatus[plugin]).map(function (key) {
                                pluginColumns.push([key, hookStatus[plugin][key]]);
                            });
                        }
                        if (process.env.HOME && plugins[plugin].indexOf(process.env.HOME) === 0) {
                            pluginColumns.push(['location', plugins[plugin].replace(process.env.HOME, '~')]);
                        }
                        else {
                            pluginColumns.push(['location', plugins[plugin]]);
                        }
                        __1.Utils.outputTable(pluginColumns, `[${plugin}]`);
                    }
                });
            }
            // application information
            if (hookStatus['application']) {
                let plugin = 'application';
                const columns = [];
                if (hookStatus && hookStatus[plugin]) {
                    Object.keys(hookStatus[plugin]).forEach(function (key) {
                        columns.push([key, hookStatus[plugin][key]]);
                    });
                }
                __1.Utils.outputTable(columns, 'Application Information');
            }
            process.exit(0);
        }
        catch (e) {
            __1.Utils.error(e.stack);
        }
    });
};
//# sourceMappingURL=status.js.map