#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const path_1 = __importDefault(require("path"));
const update_notifier_1 = __importDefault(require("update-notifier"));
// import pkg from '../package.json'
const yargs_1 = __importDefault(require("yargs"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const pkg = _1.Utils.loadCorePackageInfo();
update_notifier_1.default({ pkg, updateCheckInterval: 1000 * 60 * 60 * 24 * 7 }).notify({
    defer: false,
    isGlobal: true
});
const cache = _1.Utils.getInternalCache();
let parsedArgv = yargs_parser_1.default(process.argv.slice(2));
let parsedArgvOrigin = parsedArgv;
cache.set('argv', parsedArgv); // set argv first time
let appConfig = _1.Utils.getApplicationConfig();
yargs_1.default.config(appConfig);
parsedArgv = _1.Utils._.merge(appConfig, parsedArgv);
cache.set('argv', parsedArgv); // set argv second time
cache.set('yargs', yargs_1.default);
const plugins = _1.Utils.getAllPluginsMapping();
const config = _1.Utils.getCombinedConfig();
const packageConfig = _1.Utils.loadPackageInfo();
if (!parsedArgv.scriptName) {
    yargs_1.default.hide('script-name').option('script-name', {
        default: 'zignis',
        describe: 'Rename script name.',
        type: 'string'
    });
}
else {
    if (!_1.Utils._.isString(parsedArgv.scriptName)) {
        _1.Utils.error('--script-name must be string, should be used only once.');
    }
    yargs_1.default.scriptName(parsedArgv.scriptName);
}
yargs_1.default.hide('plugin-prefix').option('plugin-prefix', {
    default: 'zignis',
    describe: 'Set plugin prefix.'
});
let scriptName = parsedArgv.scriptName || 'zignis';
const opts = {
    // Give each command an ability to disable temporarily
    visit: (command) => {
        command.middlewares = command.middlewares ? _1.Utils._.castArray(command.middlewares) : [];
        if (command.middleware) {
            command.middlewares.concat(command.middleware);
        }
        return command.disabled === true ? false : command;
    }
};
if (!parsedArgv.disableCoreCommand) {
    // Load local commands
    yargs_1.default.commandDir('commands', opts);
}
// Load extra commands by --command-dir option
if (parsedArgv.commandDir) {
    if (parsedArgv.commandDir[0] !== '/') {
        parsedArgv.commandDir = path_1.default.resolve(process.cwd(), parsedArgv.commandDir);
    }
    if (_1.Utils.fileExistsSyncCache(path_1.default.resolve(parsedArgv.commandDir))) {
        yargs_1.default.commandDir(parsedArgv.commandDir, opts);
    }
}
// Load plugin commands
if (plugins) {
    Object.keys(plugins).map(function (plugin) {
        if (config.pluginConfigs[plugin] &&
            config.pluginConfigs[plugin].commandDir &&
            _1.Utils.fileExistsSyncCache(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir))) {
            yargs_1.default.commandDir(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir), opts);
        }
    });
}
// Load application commands
if (packageConfig.name !== scriptName &&
    appConfig.commandDir &&
    _1.Utils.fileExistsSyncCache(path_1.default.resolve(process.cwd(), appConfig.commandDir))) {
    yargs_1.default.commandDir(path_1.default.resolve(process.cwd(), appConfig.commandDir), opts);
}
;
(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        // @ts-ignore
        // Register global middlewares
        yargs_1.default.middleware((argv, yargs) => {
            const commandPath = yargs.getContext().fullCommands.slice().map(cmd => cmd.split(' ')[0]);
            let commandDefault;
            if (argv.commandDefault && commandPath.length >= 1) {
                while (commandPath.length >= 1) {
                    commandDefault = _1.Utils._.get(argv.commandDefault, commandPath);
                    if (!_1.Utils._.isObject(commandDefault) || _1.Utils._.isArray(commandDefault)) {
                        commandPath.pop();
                        continue;
                    }
                    break;
                }
            }
            // Insert home rc command default options between default options and cli options
            // So the priority is: command default options < application rc options < home rc options < cli options
            const overrideArgv = {};
            const aliases = yargs.parsed.aliases;
            Object.keys(parsedArgvOrigin).filter(key => key !== '_').forEach(key => {
                if (aliases[key] && Array.isArray(aliases[key])) {
                    overrideArgv[key] = parsedArgvOrigin[key];
                    aliases[key].forEach(alias => {
                        overrideArgv[alias] = parsedArgvOrigin[key];
                    });
                }
            });
            argv = commandDefault ? _1.Utils._.merge(argv, _1.Utils.formatRcOptions(commandDefault), overrideArgv) : argv;
            cache.set('argv', argv); // set argv third time
            return argv;
        });
        if (!parsedArgv.getYargsCompletions) {
            let beforeHooks = yield _1.Utils.invokeHook('beforeCommand');
            Object.keys(beforeHooks).map(function (hook) {
                beforeHooks[hook](parsedArgv, yargs_1.default);
            });
        }
        if (!parsedArgv.disableCoreCommand && !parsedArgv.disableCore) {
            yargs_1.default.hide('disable-core-command').option('disable-core-command', {
                alias: 'disable-core',
                describe: 'Disable core commands.'
            });
            if (!parsedArgv.disableCompletionCommand && !parsedArgv.disableCompletion) {
                yargs_1.default.hide('disable-completion-command').option('disable-completion-command', {
                    alias: 'disable-completion',
                    describe: 'Disable completion command.'
                });
                if (!parsedArgv.hideCompletionCommand && !parsedArgv.hideCompletion) {
                    yargs_1.default.hide('hide-completion-command').option('hide-completion-command', {
                        alias: 'hide-completion',
                        describe: 'Hide completion command.'
                    });
                    yargs_1.default.completion('completion', 'Generate completion script');
                }
                else {
                    // @ts-ignore, @types/yargs type def not correct
                    yargs_1.default.completion('completion', false);
                }
            }
        }
        if (!parsedArgv.disableGlobalPlugin && !parsedArgv.disableGlobalPlugins) {
            yargs_1.default.hide('disable-global-plugin').option('disable-global-plugin', {
                alias: 'disable-global-plugins',
                describe: 'Disable global plugins.'
            });
        }
        if (!parsedArgv.disableHomePlugin && !parsedArgv.disableHomePlugins) {
            yargs_1.default.hide('disable-home-plugin').option('disable-home-plugin', {
                alias: 'disable-home-plugins',
                describe: 'Disable home plugins.'
            });
        }
        if (!parsedArgv.hideEpilog) {
            yargs_1.default.hide('hide-epilog').option('hide-epilog', {
                describe: 'Hide epilog.'
            });
            yargs_1.default.hide('set-epilog').option('set-epilog', {
                default: false,
                describe: 'Set epilog.'
            });
            yargs_1.default.epilog(((epilog) => {
                if (epilog && _1.Utils._.isString(epilog)) {
                    return epilog;
                }
                else if (_1.Utils._.isArray(epilog)) {
                    let pop = epilog.pop();
                    if (pop) {
                        return pop;
                    }
                }
                return 'Find more information at https://zignis.js.org';
            })(parsedArgv.setEpilog));
        }
        if (!parsedArgv.setVersion) {
            yargs_1.default.hide('set-version').option('set-version', {
                describe: 'Set version.'
            });
        }
        else {
            yargs_1.default.version(parsedArgv.setVersion);
        }
        yargs_1.default.hide('node-env-key').option('node-env-key', {
            default: 'NODE_ENV',
            alias: 'node-env',
            describe: 'Set node env key'
        });
        // eslint-disable-next-line
        yargs_1.default
            .help()
            .alias('h', 'help')
            .exitProcess(false)
            .recommendCommands()
            .wrap(Math.min(120, yargs_1.default.terminalWidth())).argv;
        if (!parsedArgv.getYargsCompletions) {
            let afterHooks = yield _1.Utils.invokeHook('afterCommand');
            Object.keys(afterHooks).map(function (hook) {
                afterHooks[hook](parsedArgv, yargs_1.default);
            });
        }
    }
    catch (e) {
        if (!e.name || e.name !== 'YError') {
            _1.Utils.error(e.stack);
        }
    }
}))();
//# sourceMappingURL=main.js.map